#nawk-script for the evaluation of ParixAdm-Configurations
#Parsytec Computer GmbH, VHO, 10.1.91
#SCCSid:	@(#)evalconfig.nawk	1.1	5/15/92
#
#This script evaluates the configuration files and performs
#actions depending on its arguments:
#- consistency check: is the state of the system reasonable ?
#- free partitions: print list of partitions available
#- allocation table: print table with use of partitions
#- configuration table: print table of configuration
#
#Parameters:
#nawk -f evalconfig.nawk Entry.config.file State.file DefaultAttrib
	command [Parameter]
#where:
#   -f evalconfig.nawk tells nawk the name of this program
#   DefaultAttrib is currently ignored.
#   Entry.config.file is the name of the configuration file
#   State.file is the name of the file that contains the used partitions
#   Command is the action to be taken:
#       c: constistency check. Output are possible errors, exit-
#	   value is =0 if no errors, !=0 otherwise
#       f: print a list of free partitions. Command f has two
#          mandatory parameters, the name of a partition or an
#          attribute that the required partition has and the command
#          to be executed.
#          Note: This list can be empty !
#       a: print a table of partitions with information about
#          who uses what etc.
#       t: print a summary of the configuration-file
#Return value:
#  0 if everything worked as intended
#  !=0 in case of an error
#
#{print $0}	#Debugging#
#
#Print Error end exit with ExitCode
func PrintErr(Message,code){
	print ARGV[0]":"Message":line "FNR" of "FILENAME
	error=code
	exit(code)
}	#func PrintErr

#Variables set in BEGIN:
# DefaultAttrib: currently ignored
# Command: Command character
# Attrib: desired attrib with command c
# CLine: desired command in Partition
#
BEGIN{
	error=0	#error flag
	#
	#argument processing
	#
	if(ARGC <5){
		PrintErr("need at least 5 parameters",1)
	}	#if <5
	DefaultAttrib=ARGV[3]
	Command=ARGV[4];
	if (Command=="f"){
		if (ARGC!=7){
			PrintErr("command f needs 7 parameters",2)
		}
		Attribute=ARGV[5]
		CLine=ARGV[6]
	} else if (Command ~ /c|t|a/){
		if (ARGC!=5){
			PrintErr("command a,t,f need 5 parameters",3)
		}	#if error
	} else {
		PrintErr("unknown command",4)
	}	#if command
	ARGC=3	#use only the first 2 filenames
	#
	#set field separator to :
	#
	FS=":"
	OFS=":"
}	#BEGIN

#add element entry to list. list is a FS-separated string with no
#FS in front or end. Note: entry must not contain FS.
#Result: List with added element
func AddList(list,entry){
	if (entry ~ FS){
		PrintErr("AddList with FS",18)
	}	#error
	if (list==""){
		list=entry
	}else{
		list=list FS entry
	}
	return list
}	#func AddList

#The following instructions build tables with information, that are
#used in the "postprocessing" at END.
#Variables:
#	ADM: administrator id + comment
#	ENTRIES[]:Index is EntryName, contents is ""
#	PARTITIONS[]:Index is PartitionName, contents is list of entries
#	ATTRIBS[]:Index is Attribute, contents is list of partitions
#	PARCOMMANDS:Index is [Partition,Command], contents is ""
#	USERS:Index is Partition, contents is full line
#	EntryPart: Index is Entry, contents is list of partitions using
#		this entry
#	PartAvail: Partitions that are available, Index is PartitionName,
#		contents is "" if available, list of colliding partitions otherwise
#	PartComment: Comment of Partition. Index is PartitionName
#	PartCommand: List of Commands for Partition.
#	PartAttrib: Attributes per Partition
#Temporary Variables
#	Fields: to split line in USER
#	P: current Partition in USER
#	i,j: counter
#	FirstFlag: 0 if something is happening for the first time
#	UEntries: array of used entries in User
#	UEntry: current used entry
#	UPartitions: array of used partitions
#	UPartition: current used partition
#

func debug(){
	print "ADM=" ADM
	print "Entries are"
	for (i in ENTRIES) print "\t["i"]=" ENTRIES[i]
	print "Partitions are"
	for (i in PARTITIONS) print "\t["i"]=" PARTITIONS[i]
	print "Attribs are"
	for (i in ATTRIBS) print "\t["i"]=" ATTRIBS[i]
	print "ParCommands are"
	for (i in PARCOMMANDS) print "\t["i"]=" PARCOMMANDS[i]
	print "Users are"
	for (i in USERS) print "\t["i"]=" USERS[i]
	print "EntryPart are"
	for (i in EntryPart) print "\t["i"]=" EntryPart[i]
	print "PartAvail are"
	for (i in PartAvail) print "\t["i"]=" PartAvail[i]
}	#debug

/^#/{	## comment
	#This is a comment. Don't do anything
	next
}	#comment

/^ADM:/{	#ADM:id:comment
	if (NF!=3){
		PrintErr("ADM must have 3 fields",5)
	}	#error
	if (ADM != ""){
		PrintErr("ADM already defined",6)
	}	#error
	ADM=$2 FS $3
	next
}	#ADM

/^ENTRY:/{	#ENTRY:LogName:PhysName:hostname:comment
	if (NF!=5){
		PrintErr("ENTRY must have 5 fields",7)
	}	#error
	if ($2 in ENTRIES){
		PrintErr("ENTRY defined twice",8)
	}	#error
	ENTRIES[$2]=""
	next
}

/^PARTITION:/{	#PARTITION:Name:comment:EntryName:...
	if (NF<4){
		PrintErr("PARTITION must have >=4 fields",9)
	}	#error
	if ($2 in PARTIONS){
		PrintErr("PARTITION defined twice",10)
	}	#error
	i=length($1 FS $2 FS $3 FS)
	PARTITIONS[$2]=substr($0,i + 1,length($0)-i)
	for(i=4;i<=NF;i++){
		if (! ($(i) in ENTRIES)){
			PrintErr("Undefined Entry in PARTITION",11)
		}	#error
		EntryPart[$(i)]=AddList(EntryPart[$(i)],$2)
	}	#for
	#Enter standard attributes
	if ($2 in ATTRIBS){
		PrintErr("Partition name collides with attribute",14)
	}	#error
	ATTRIBS[$2]=$2
	#Enter PartAvail
	PartAvail[$2]=""
	#Enter PartComment
	PartComment[$2]=$3
	next
}	#PARTITION

/^ATTRIB:/{	#ATTRIB:name:PartitionName:...
	if (NF<3){
		PrintErr("ATTRIB must have >=3 fields",12)
	}	#error
	if ($2 in ATTRIBS){
		PrintErr("ATTRIB defined twice",13)
	}	#error
	for(i=3;i<=NF;i++){
		if (! ($(i) in PARTITIONS)){
			PrintErr("Undefined Partition in ATTRIB",14)
		}	#error
		PartAttrib[$(i)]=AddList(PartAttrib[$(i)],$2)
	}	#for
	i=length($1 FS $2 FS)
	ATTRIBS[$2]=substr($0,i + 1,length($0)-i)
	next
}	#ATTRIB

/^PARCOMMAND:/{	#PARCOMMAND:PartitionName:Command:CommandLine
	if (NF!=4){
		PrintErr("PARCOMMAND must have 4 fields",15)
	}	#error
	if (! ($2 in PARTITIONS)){
		PrintErr("Undefined Partition in PARCOMMAND",22)
	}	#error
	if ($2 SUBSEP $3 in PARCOMMANDS){
		PrintErr("PARCOMMAND defined twice",13)
	}	#error
	PARCOMMANDS[$2,$3]=""
	PartCommand[$2]=AddList(PartCommand[$2],$3)
	next
}	#PARCOMMAND

/^USER&/{	# USER|Partition|whoami|date|who am i
	if(split($0,Fields,"&")!=5){
		PrintErr("USER must have 5 fields separated by &",15)
	}	#error
	P=Fields[2]
	if(!(P in PARTITIONS)){
		PrintErr("Unknown Partition for USER",16)
	}	#error
	if(P in USERS){
		PrintErr("Partition used twice by USER",17)
	}	#error
	if (PartAvail[P] != ""){
		PrintErr("Partition already blocked (USER)",18)
	}	#error
	USERS[P]=$0
	#enter data in PartAvail
	split(PARTITIONS[P],UEntries)	#get list of Entries
	for(i in UEntries){
		UEntry=UEntries[i]
		split(EntryPart[UEntry],UPartitions)
		for (j in UPartitions){
			UPartition=UPartitions[j]
			if (UPartition!=P){
				PartAvail[UPartition]=AddList(PartAvail[UPartition],P)
			}	#if myself
		}	#for UPartition
	}	#for UEntry
	PartAvail[P]=P	#myself
	next
}	#USER

#Any other line drops in here
	{	#any other case
		if(length($0)!=0){	#allow for empty lines
			PrintErr("illegal line",19);
		}	#error
	next
}	#else

END{
	if (error==0){
		if (ADM==""){
			PrintErr("no administrator set",23)
		}
		#Perform some PostProcessing depending on the Command chosen
		if (Command=="c"){	#c command
			# Checks already done
		}else if (Command=="f"){	#f command
			if (! (Attribute in ATTRIBS)){
				PrintErr("Unknown attribute as parameter", 20)
			}	#error
			split(ATTRIBS[Attribute],UPartitions)
			FirstFlag=0
			for (i in UPartitions){
				UPartition=UPartitions[i]
				if((PartAvail[UPartition]=="") && (UPartition SUBSEP CLine in PARCOMMANDS)){
	 				if (FirstFlag!=0){
						printf ":"
					}else{
						FirstFlag=1
					}	#if FirstFlag
					printf "%s", UPartition
				}	#if available
			}	#for right attribute
			print
		}else if (Command=="a"){	#a command
			print "Partition | Used by                | since" |"cat"
			print "----------+------------------------+-----------------------------" |"cat"
			close("cat")
			for (UPartition in PartAvail){
				if ("" == PartAvail[UPartition]){
					printf "%-10s| %-23s|\n",UPartition,"*" |"sort"
				}else if (PartAvail[UPartition]!=UPartition){
					printf "%-10s| %-23s|\n",UPartition,PartAvail[UPartition] |"sort"
				}else{	#used by itself
					split(USERS[UPartition],Fields,"&")
					printf "%-10s| %-23s| %s\n",UPartition,"User "Fields[3], Fields[4] |"sort"
				}	#if Partition used
			}	#for Partition
			close("sort")
		}else if (Command=="t"){	#t command
			#print table of Partitions, their Attributes, Commands,
			#Entries and comments
			print "Partition | Attributes             | Commands" |"cat"
			print "----------+------------------------+-----------------------------" |"cat"
			close("cat")
			for (UPartition in PARTITIONS){
				printf "%-10s| %-23s| %-21s\n", UPartition, PartAttrib[UPartition], PartCommand[UPartition] |"sort"
			}	#for partitions
			close("sort")
			print |"cat"
			print "Partition | Entries                | Comments" |"cat"
			print "----------+------------------------+-----------------------------" |"cat"
			close("cat")
			for (UPartition in PARTITIONS){
				printf "%-10s| %-23s| %-21s\n", UPartition, PARTITIONS[UPartition], PartComment[UPartition] |"sort"
			}	#for partitions
			close("sort")
			print "Your administrator is " ADM |"cat"
			close("cat")
		}else{
			PrintErr("Programmer mad",22)
		}	#if Command
	}	#if !error
}	#END
